import math

def linear_trans(arr,x):
    return [sum(map(lambda x: x[0]*x[1],zip(line,x))) for line in arr]


def rotate_line(line,p,dgr):
    '''
    Rotate y=kx+b with p for dgr.
    dgr measured in radians.
    @param line:
        tuple[k, b] => y = kx + b
    
    '''
    k,b = line
    arr = [
        [math.cos(dgr),-math.sin(dgr)],
        [math.sin(dgr), math.cos(dgr)]
    ]
    kvec  = [1, k]
    rkvec = linear_trans(arr,kvec)
    rk = rkvec[1]/rkvec[0] # (a,b) => (1,b/a)
    
    p1 = [0,b]
    pvec = [p1[0]-p[0],p1[1]-p[1]]
    rpvec = linear_trans(arr,pvec)
    rp1 = [p[0] + rpvec[0], p[1]+rpvec[1]]
    
    return [rk,rp1[1]-rk*rp1[0]]

import numpy as np
def rotate_line_np(line,p:np.ndarray,dgr) -> np.ndarray:
    k,b = line
    arr = np.array([[np.cos(dgr),-np.sin(dgr)],[np.sin(dgr),np.cos(dgr)]])
    vec = np.dot(arr,[1,k])
    rk  = vec[1]/vec[0]
    
    p1 = np.array([0,b])
    rp = np.dot(arr, p1-p) + p
    
    return np.array([rk,rp[1]-rk*rp[0]])

print(rotate_line((3,4),(2,1),math.pi/4))
print(rotate_line_np((3,4),np.array([2,1]),math.pi/4))